// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package otelhttptrace

// Version is the current release version of the httptrace instrumentation.
const Version = "0.70.0"
