// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package internal

// Version is the current release version of the OpenTelemetry OTLP HTTP trace
// exporter in use.
const Version = "1.45.0"
