// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package metric

// version is the current release version of the metric SDK in use.
func version() string {
	return "1.45.0"
}
